@@ -27,23 +27,23 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 """
 
-import re
-from functools import reduce, wraps
-import operator
-import sys
-import logging
-from typing import (
-        Any, Callable, Dict, Generic, Hashable, Iterable,
-        List, Optional, Set, Tuple, TypeVar, Union, ClassVar)
 import builtins
-
+import logging
 import math
+import operator
+import re
+import sys
+from functools import reduce, wraps
 from sys import intern
+from typing import (
+    Any, Callable, ClassVar, Dict, Generic, Hashable, Iterable, List, Mapping,
+    Optional, Set, Tuple, Type, TypeVar, Union)
+
 
 try:
-    from typing import SupportsIndex, Concatenate
+    from typing import Concatenate, SupportsIndex
 except ImportError:
-    from typing_extensions import SupportsIndex, Concatenate
+    from typing_extensions import Concatenate, SupportsIndex
 
 try:
     from typing import ParamSpec
@@ -1584,6 +1584,8 @@ def get_write_to_map_from_permutation(original, permuted):
 # {{{ graph algorithms
 
 from pytools.graph import a_star as a_star_moved
+
+
 a_star = MovedFunctionDeprecationWrapper(a_star_moved)
 
 # }}}
@@ -1871,7 +1873,7 @@ def string_histogram(  # pylint: disable=too-many-arguments,too-many-locals
                 print(value, bin_nr, bin_starts)
                 raise
 
-    from math import floor, ceil
+    from math import ceil, floor
     if use_unicode:
         def format_bar(cnt):
             scaled = cnt*width/max_count
@@ -2014,7 +2016,24 @@ class StderrToStdout:
         del self.stderr_backup
 
 
-def typedump(val, max_seq=5, special_handlers=None):
+def typedump(val: Any, max_seq: int = 5,
+             special_handlers: Optional[Mapping[Type, Callable]] = None,
+             fully_qualified_name: bool = True) -> str:
+    """
+    Return a string representation of the type of *val*, recursing into
+    iterable objects.
+
+    :arg val: The object for which the type should be returned.
+    :arg max_seq: For iterable objects, the maximum number of elements to
+        include in the return string. Lower this value if you get a
+        :class:`RecursionError`.
+    :arg special_handlers: An optional mapping of specific types to special
+        handlers.
+    :arg fully_qualified_name: Return fully qualified names, that is,
+        include module names and use ``__qualname__`` instead of ``__name__``.
+
+    :returns: A string representation of the type of *val*.
+    """
     if special_handlers is None:
         special_handlers = {}
 
@@ -2025,10 +2044,26 @@ def typedump(val, max_seq=5, special_handlers=None):
     else:
         return hdlr(val)
 
+    def objname(obj: Any) -> str:
+        if type(obj).__module__ == "builtins":
+            if fully_qualified_name:
+                return type(obj).__qualname__
+            else:
+                return type(obj).__name__
+
+        if fully_qualified_name:
+            return type(obj).__module__ + "." + type(obj).__qualname__
+        else:
+            return type(obj).__name__
+
+    # Special handling for 'str' since it is also iterable
+    if isinstance(val, str):
+        return "str"
+
     try:
         len(val)
     except TypeError:
-        return type(val).__name__
+        return objname(val)
     else:
         if isinstance(val, dict):
             return "{%s}" % (
@@ -2038,17 +2073,16 @@ def typedump(val, max_seq=5, special_handlers=None):
 
         try:
             if len(val) > max_seq:
-                return "{}({},...)".format(
-                        type(val).__name__,
-                        ",".join(typedump(x, max_seq, special_handlers)
-                            for x in val[:max_seq]))
+                t = ",".join(typedump(x, max_seq, special_handlers)
+                            for x in val[:max_seq])
+                return f"{objname(val)}({t},...)"
             else:
-                return "{}({})".format(
-                        type(val).__name__,
-                        ",".join(typedump(x, max_seq, special_handlers)
-                            for x in val))
+                t = ",".join(typedump(x, max_seq, special_handlers)
+                            for x in val)
+                return f"{objname(val)}({t})"
+
         except TypeError:
-            return val.__class__.__name__
+            return objname(val)
 
 
 def invoke_editor(s, filename="edit.txt", descr="the file"):
@@ -2169,7 +2203,7 @@ def assert_not_a_file(name):
 
 
 def add_python_path_relative_to_script(rel_path):
-    from os.path import dirname, join, abspath
+    from os.path import abspath, dirname, join
 
     script_name = sys.argv[0]
     rel_script_dir = dirname(script_name)
@@ -2391,8 +2425,9 @@ def download_from_web_if_not_present(url, local_name=None):
         local_name = basename(url)
 
     if not exists(local_name):
-        from pytools.version import VERSION_TEXT
         from urllib.request import Request, urlopen
+
+        from pytools.version import VERSION_TEXT
         req = Request(url, headers={
             "User-Agent": f"pytools/{VERSION_TEXT}"
             })
@@ -2412,7 +2447,7 @@ def find_git_revision(tree_root):  # pylint: disable=too-many-locals
     # Keep this routine self-contained so that it can be copy-pasted into
     # setup.py.
 
-    from os.path import join, exists, abspath
+    from os.path import abspath, exists, join
     tree_root = abspath(tree_root)
 
     if not exists(join(tree_root, ".git")):
@@ -2432,7 +2467,7 @@ def find_git_revision(tree_root):  # pylint: disable=too-many-locals
     env["LANG"] = "C"
     env["LC_ALL"] = "C"
 
-    from subprocess import Popen, PIPE, STDOUT
+    from subprocess import PIPE, STDOUT, Popen
     p = Popen(["git", "rev-parse", "HEAD"], shell=False,
               stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True,
               cwd=tree_root, env=env)
@@ -2536,6 +2571,18 @@ class ProcessTimer:
 
 # {{{ log utilities
 
+def _log_start_if_long(logger, sleep_duration, done_indicator,
+                       noisy_level, description):
+    from time import sleep
+    sleep(sleep_duration)
+
+    if not done_indicator[0]:
+        logger.log(
+                noisy_level, "%s: started %.gs ago",
+                description,
+                sleep_duration)
+
+
 class ProcessLogger:  # pylint: disable=too-many-instance-attributes
     """Logs the completion time of a (presumably) lengthy process to :mod:`logging`.
     Only uses a high log level if the process took perceptible time.
@@ -2560,13 +2607,16 @@ class ProcessLogger:  # pylint: disable=too-many-instance-attributes
                 0.3 if long_threshold_seconds is None else long_threshold_seconds)
 
         self.logger.log(self.silent_level, "%s: start", self.description)
-        self.is_done = False
+        self._done_indicator = [False]
 
         import threading
-        self.late_start_log_thread = threading.Thread(target=self._log_start_if_long)
+        self.late_start_log_thread = threading.Thread(
+                target=_log_start_if_long,
+                args=(logger, 10*self.long_threshold_seconds, self._done_indicator,
+                      self.noisy_level, self.description),
 
-        # Do not delay interpreter exit if thread not finished.
-        self.late_start_log_thread.daemon = True
+                # Do not delay interpreter exit if thread not finished.
+                daemon=True)
 
         # https://github.com/firedrakeproject/firedrake/issues/1422
         # Starting a thread may irrecoverably break various environments,
@@ -2604,22 +2654,10 @@ class ProcessLogger:  # pylint: disable=too-many-instance-attributes
 
         self.timer = ProcessTimer()
 
-    def _log_start_if_long(self):
-        from time import sleep
-
-        sleep_duration = 10*self.long_threshold_seconds
-        sleep(sleep_duration)
-
-        if not self.is_done:
-            self.logger.log(
-                    self.noisy_level, "%s: started %.gs ago",
-                    self.description,
-                    sleep_duration)
-
     def done(  # pylint: disable=keyword-arg-before-vararg
             self, extra_msg=None, *extra_fmt_args):
         self.timer.done()
-        self.is_done = True
+        self._done_indicator[0] = True
 
         completion_level = (
                 self.noisy_level
@@ -2808,8 +2846,8 @@ def unordered_hash(hash_instance, iterable, hash_constructor=None):
     """
 
     if hash_constructor is None:
-        from functools import partial
         import hashlib
+        from functools import partial
         hash_constructor = partial(hashlib.new, hash_instance.name)
 
     h_int = 0
