@@ -21,11 +21,14 @@ THE SOFTWARE.
 """
 
 
+import logging
 import sys
+
 import pytest
 
-import logging
+
 logger = logging.getLogger(__name__)
+from typing import FrozenSet
 
 
 @pytest.mark.skipif("sys.version_info < (2, 5)")
@@ -49,14 +52,14 @@ def test_memoize_method_clear():
     sc.f.clear_cache(sc)  # pylint: disable=no-member
 
 
-def test_memoize_method_with_uncached():
-    from pytools import memoize_method_with_uncached
+def test_keyed_memoize_method_with_uncached():
+    from pytools import keyed_memoize_method
 
     class SomeClass:
         def __init__(self):
             self.run_count = 0
 
-        @memoize_method_with_uncached(uncached_args=[1], uncached_kwargs=["z"])
+        @keyed_memoize_method(key=lambda x, y, z: x)
         def f(self, x, y, z):
             del x, y, z
             self.run_count += 1
@@ -170,6 +173,7 @@ def test_memoize_keyfunc():
 
 def test_memoize_frozen():
     from dataclasses import dataclass
+
     from pytools import memoize_method
 
     # {{{ check frozen dataclass
@@ -272,7 +276,7 @@ def test_find_module_git_revision():
 
 def test_reshaped_view():
     import pytools
-    import numpy as np
+    np = pytest.importorskip("numpy")
 
     a = np.zeros((10, 2))
     b = a.T
@@ -296,6 +300,7 @@ def test_processlogger():
 
 def test_table():
     import math
+
     from pytools import Table
 
     tbl = Table()
@@ -308,11 +313,23 @@ def test_table():
     print()
     print(tbl.latex())
 
+    # {{{ test merging
+
+    from pytools import merge_tables
+    tbl = merge_tables(tbl, tbl, tbl, skip_columns=(0,))
+    print(tbl.github_markdown())
+
+    # }}}
+
 
 def test_eoc():
+    np = pytest.importorskip("numpy")
+
     from pytools.convergence import EOCRecorder
     eoc = EOCRecorder()
 
+    # {{{ test pretty_print
+
     for i in range(1, 8):
         eoc.add_data_point(1.0 / i, 10 ** (-i))
 
@@ -326,16 +343,48 @@ def test_eoc():
             eoc_format="%5.2f")
     print(p)
 
+    # }}}
+
+    # {{{ test merging
+
+    from pytools.convergence import stringify_eocs
+    p = stringify_eocs(eoc, eoc, eoc, names=("First", "Second", "Third"))
+    print(p)
+
+    # }}}
+
+    # {{{ test invalid inputs
+
+    import numpy as np
+
+    eoc = EOCRecorder()
+
+    # scalar inputs are fine
+    eoc.add_data_point(1, 1)
+    eoc.add_data_point(1.0, 1.0)
+    eoc.add_data_point(np.float32(1.0), 1.0)
+    eoc.add_data_point(np.array(3), 1.0)
+    eoc.add_data_point(1.0, np.array(3))
+
+    # non-scalar inputs are not fine though
+    with pytest.raises(TypeError):
+        eoc.add_data_point(np.array([3]), 1.0)
+
+    with pytest.raises(TypeError):
+        eoc.add_data_point(1.0, np.array([3]))
+
+    # }}}
+
 
 def test_natsorted():
-    from pytools import natsorted, natorder
+    from pytools import natorder, natsorted
 
     assert natorder("1.001") < natorder("1.01")
 
     assert natsorted(["x10", "x1", "x9"]) == ["x1", "x9", "x10"]
     assert natsorted(map(str, range(100))) == list(map(str, range(100)))
     assert natsorted(["x10", "x1", "x9"], reverse=True) == ["x10", "x9", "x1"]
-    assert natsorted([10, 1, 9], key=lambda d: "x%d" % d) == [1, 9, 10]
+    assert natsorted([10, 1, 9], key=lambda d: f"x{d}") == [1, 9, 10]
 
 
 # {{{ object array iteration behavior
@@ -354,6 +403,8 @@ class FakeArray:
 
 
 def test_make_obj_array_iteration():
+    pytest.importorskip("numpy")
+
     from pytools.obj_array import make_obj_array
     make_obj_array([FakeArray()])
 
@@ -409,7 +460,7 @@ def test_obj_array_vectorize(c=1):
 
     # {{{ check
 
-    scalar_ary = np.ones(42, dtype=np.float)
+    scalar_ary = np.ones(42, dtype=np.float64)
     object_ary = obj.make_obj_array([scalar_ary, scalar_ary, scalar_ary])
 
     for func, vectorizer, nargs in [
@@ -444,14 +495,13 @@ def test_obj_array_vectorize(c=1):
 
 def test_tag():
     from pytools.tag import (
-        Taggable, Tag, UniqueTag, NonUniqueTagError, check_tag_uniqueness
-    )
+        NonUniqueTagError, Tag, Taggable, UniqueTag, check_tag_uniqueness)
 
     # Need a subclass that defines the copy function in order to test.
     class TaggableWithCopy(Taggable):
 
-        def copy(self, **kwargs):
-            return TaggableWithCopy(kwargs["tags"])
+        def _with_new_tags(self, tags):
+            return TaggableWithCopy(tags)
 
     class FairRibbon(Tag):
         pass
@@ -516,7 +566,7 @@ def test_tag():
                                 blue_ribbon, red_ribbon))
 
     # Test that tagged() fails if a UniqueTag of the same subclass
-    # is alredy present
+    # is already present
     with pytest.raises(NonUniqueTagError):
         t1.tagged(best_in_show_ribbon)
 
@@ -534,8 +584,8 @@ def test_tag():
 
 
 def test_unordered_hash():
-    import random
     import hashlib
+    import random
 
     # FIXME: Use randbytes once >=3.9 is OK
     lst = [bytes([random.randrange(256) for _ in range(20)])
@@ -555,6 +605,163 @@ def test_unordered_hash():
             != unordered_hash(hashlib.sha256(), lst).digest())
 
 
+# {{{ sphere sampling
+
+@pytest.mark.parametrize("sampling", [
+    "equidistant", "fibonacci", "fibonacci_min", "fibonacci_avg",
+    ])
+def test_sphere_sampling(sampling, visualize=False):
+    from functools import partial
+
+    from pytools import sphere_sample_equidistant, sphere_sample_fibonacci
+
+    npoints = 128
+    radius = 1.5
+
+    if sampling == "equidistant":
+        sampling_func = partial(sphere_sample_equidistant, r=radius)
+    elif sampling == "fibonacci":
+        sampling_func = partial(
+                sphere_sample_fibonacci, r=radius, optimize=None)
+    elif sampling == "fibonacci_min":
+        sampling_func = partial(
+                sphere_sample_fibonacci, r=radius, optimize="minimum")
+    elif sampling == "fibonacci_avg":
+        sampling_func = partial(
+                sphere_sample_fibonacci, r=radius, optimize="average")
+    else:
+        raise ValueError(f"unknown sampling method: '{sampling}'")
+
+    np = pytest.importorskip("numpy")
+    points = sampling_func(npoints)
+    assert np.all(np.linalg.norm(points, axis=0) < radius + 1.0e-15)
+
+    if not visualize:
+        return
+
+    import matplotlib.pyplot as plt
+    fig = plt.figure(figsize=(10, 10), dpi=300)
+    ax = fig.add_subplot(111, projection="3d")
+
+    import matplotlib.tri as mtri
+    theta = np.arctan2(np.sqrt(points[0]**2 + points[1]**2), points[2])
+    phi = np.arctan2(points[1], points[0])
+    triangles = mtri.Triangulation(theta, phi)
+
+    ax.plot_trisurf(points[0], points[1], points[2], triangles=triangles.triangles)
+    ax.set_xlim([-radius, radius])
+    ax.set_ylim([-radius, radius])
+    ax.set_zlim([-radius, radius])
+    ax.margins(0.05, 0.05, 0.05)
+
+    # plt.show()
+    fig.savefig(f"sphere_sampling_{sampling}")
+    plt.close(fig)
+
+# }}}
+
+
+def test_unique_name_gen_conflicting_ok():
+    from pytools import UniqueNameGenerator
+
+    ung = UniqueNameGenerator()
+    ung.add_names({"a", "b", "c"})
+
+    with pytest.raises(ValueError):
+        ung.add_names({"a"})
+
+    ung.add_names({"a", "b", "c"}, conflicting_ok=True)
+
+
+def test_ignoredforequalitytag():
+    from pytools.tag import IgnoredForEqualityTag, Tag, Taggable
+
+    # Need a subclass that defines _with_new_tags in order to test.
+    class TaggableWithNewTags(Taggable):
+
+        def _with_new_tags(self, tags: FrozenSet[Tag]):
+            return TaggableWithNewTags(tags)
+
+    class Eq1(IgnoredForEqualityTag):
+        pass
+
+    class Eq2(IgnoredForEqualityTag):
+        pass
+
+    class Eq3(Tag):
+        pass
+
+    eq1 = TaggableWithNewTags(frozenset([Eq1()]))
+    eq2 = TaggableWithNewTags(frozenset([Eq2()]))
+    eq12 = TaggableWithNewTags(frozenset([Eq1(), Eq2()]))
+    eq3 = TaggableWithNewTags(frozenset([Eq1(), Eq3()]))
+
+    assert eq1 == eq2 == eq12
+    assert eq1 != eq3
+
+    assert eq1.without_tags(Eq1())
+    with pytest.raises(ValueError):
+        eq3.without_tags(Eq2())
+
+    assert hash(eq1) == hash(eq2) == hash(eq12)
+    assert hash(eq1) != hash(eq3)
+
+
+def test_strtobool():
+    from pytools import strtobool
+    assert strtobool("true") is True
+    assert strtobool("tRuE") is True
+    assert strtobool("1") is True
+    assert strtobool("t") is True
+    assert strtobool("on") is True
+
+    assert strtobool("false") is False
+    assert strtobool("FaLse") is False
+    assert strtobool("0") is False
+    assert strtobool("f") is False
+    assert strtobool("off") is False
+
+    with pytest.raises(ValueError):
+        strtobool("tru")
+        strtobool("fal")
+        strtobool("xxx")
+        strtobool(".")
+
+    assert strtobool(None, False) is False
+
+
+def test_typedump():
+    from pytools import typedump
+    assert typedump("") == "str"
+    assert typedump("abcdefg") == "str"
+    assert typedump(5) == "int"
+
+    assert typedump((5.0, 4)) == "tuple(float,int)"
+    assert typedump([5, 4]) == "list(int,int)"
+    assert typedump({5, 4}) == "set(int,int)"
+    assert typedump(frozenset((1, 2, 3))) == "frozenset(int,int,int)"
+
+    assert typedump([5, 4, 3, 2, 1]) == "list(int,int,int,int,int)"
+    assert typedump([5, 4, 3, 2, 1, 0]) == "list(int,int,int,int,int,...)"
+    assert typedump([5, 4, 3, 2, 1, 0], max_seq=6) == "list(int,int,int,int,int,int)"
+
+    assert typedump({5: 42, 7: 43}) == "{'5': int, '7': int}"
+
+    class C:
+        class D:
+            pass
+
+    assert typedump(C()) == "test_pytools.test_typedump.<locals>.C"
+    assert typedump(C.D()) == "test_pytools.test_typedump.<locals>.C.D"
+    assert typedump(C.D(), fully_qualified_name=False) == "D"
+
+    from pytools.datatable import DataTable
+    t = DataTable(column_names=[])
+
+    assert typedump(t) == "pytools.datatable.DataTable()"
+    assert typedump(t, special_handlers={type(t): lambda x: "foo"}) == "foo"
+
+
 if __name__ == "__main__":
     if len(sys.argv) > 1:
         exec(sys.argv[1])
